#ifndef UTIL_H
#define UTIL_H

#include <utility>
#include <chrono>

// A _very crude_ timer implementation. Do not use this in production code. It uses
// std::chrono for timekeeping, you can look up that library's documentation if you
// want but the implementation of this class is outside the scope of CS 32.
class Timer {
  using duration = std::chrono::high_resolution_clock::duration;
  
  std::chrono::high_resolution_clock::time_point tick_;
   duration elapsed_time_;
public:
  Timer() : tick_(std::chrono::high_resolution_clock::now()), elapsed_time_(0) {}
  
  void Tick() {
    tick_ = std::chrono::high_resolution_clock::now();
  }
  
  void Tock() {
    auto tock = std::chrono::high_resolution_clock::now();
    elapsed_time_ += tock - tick_;
  }

  // Resets the tick and the elapsed time
  void Reset() {
    tick_ = std::chrono::high_resolution_clock::now();
    elapsed_time_ = duration(0);
  }
  
  
  // Returns the total elapsed time between ticks and tocks as
  // milliseconds.
  double ElapsedTime() const {
    return double(elapsed_time_.count()) / (double(duration::period::den) / 1000);
  }
};

#endif // UTIL_H
